###########################
#  AdministratorPersonal  #
###########################

##########
#
#  Copyright 2002, Tom Hoffman.
# 
#  This file is part of Tuttle.
#
#  Tuttle is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  Tuttle is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Tuttle (see COPYING.txt); if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##########

from OFS.Folder import Folder
from Globals import DTMLFile, InitializeClass
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from AccessControl import ClassSecurityInfo

class AdministratorPersonal(CatalogAware, Folder):

	"The basic container for a administrator's objects, with some basic attributes"

	meta_type = "administrator"
	
	_properties=(
	{'id':'title','type':'string'},
	{'id':'FirstName','type':'string'},
	{'id':'LastName','type':'string'},
	{'id':'MiddleName','type':'string'},
	{'id':'PreferredName','type':'string'},
	{'id':'Prefix','type':'string'},
	{'id':'Suffix','type':'string'},
	{'id':'Title','type':'string'},
	{'id':'Gender','type':'selection','select_variable':'Genders'},
	)
	
	manage_options=Folder.manage_options[:3] + (
        # future additions here
	) + Folder.manage_options[3:]



	def __init__ (self, id, FirstName, LastName, MiddleName="", PreferredName="", \
	Prefix="", Suffix="", Title="", Gender="?", REQUEST=None):
		self.id=id
		self.editAdministratorPersonal(FirstName, LastName, MiddleName, PreferredName, \
		Prefix, Suffix, Title, Gender)

        # set up security boilerplate
        security = ClassSecurityInfo()
        # security.setDefaultAccess("deny")
        # __roles__ = ()

	security.declareProtected('Manage School', 'editAdministratorPersonal')
	def editAdministratorPersonal (self, FirstName, LastName, MiddleName, PreferredName, \
	Prefix, Suffix, Title, Gender, REQUEST=None):
		"Method for updating administrator personal data"
		self.FirstName=FirstName
		self.MiddleName=MiddleName
		self.LastName=LastName
		self.PreferredName=PreferredName
		if self.title[-7:] != 'Files':
			if self.PreferredName != "":
				self.title=PreferredName+"\'s Files"
			else:
				self.title=FirstName+"\'s Files"
		self.Suffix=Suffix
		self.Prefix=Prefix
		self.Gender=Gender
		self.Title=Title
		self.reindex_object()
		if REQUEST is not None:
			return self.index_html(self,REQUEST)




	# Web Presentation Methods
	security.declareProtected('View', 'index_html')
	index_html=DTMLFile("dtml/indexAdministratorPersonal",globals())
InitializeClass(AdministratorPersonal)
