################
#  SchoolInfo  #
################

###########
#
#  Copyright 2002, Tom Hoffman.
# 
#  This file is part of Tuttle.
#
#  Tuttle is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  Tuttle is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Tuttle (see COPYING.txt); if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##########

from OFS.Folder import Folder
from Globals import DTMLFile, MessageDialog, InitializeClass
from AccessControl.User import UserFolder
from AccessControl import ClassSecurityInfo
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from StudentsFolder import StudentsFolder
from TeachersFolder import TeachersFolder
from Team import Team
from AdministratorsFolder import AdministratorsFolder
# from plugins import Plugins (coming in 0.2)
from zLOG import LOG, INFO
from Day import Day
import string
from whrandom import choice
import non_sequitur
import pdfReportLab

def manage_addSchoolInfo(self,id,title,\
district,superintendent,REQUEST):
	"Method for adding a Tuttle site"
	newSchool = SchoolInfo(id,title,district,superintendent)
	# add a Zope User Folder
	userFolder = UserFolder()
	newSchool._setObject('acl_users', userFolder)
        # add a plugin registry folder
	#plug = Plugins('plugins','Tuttle Product Plugin Registry')
	#newSchool._setObject('plugins', plug)
	# add necessary roles
	newSchool._addRole('teacher')
	newSchool._addRole('student')
	newSchool._addRole('student-owner') # local role for a student's own info
	newSchool._addRole('administrator')
	newSchool._addRole('parent') # 'parent' is a generic schoolwide role
	newSchool._addRole('guardian') # 'guardian' is only assigned as a local role
                                       # for their individual student's folder
	# add a folder for students
	studFolder=StudentsFolder('student', title + ' Student Records')
	newSchool._setObject('student',studFolder)
	# add a folder for teachers
	teachFolder=TeachersFolder('teacher', title + ' Teacher Files')
	newSchool._setObject('teacher',teachFolder)
	# add a folder for administrators
	adminFolder=AdministratorsFolder('administrator', title + ' Administration Files')
        newSchool._setObject('administrator',adminFolder)






        # Finally, set the SchoolInfo object!
	self._setObject(id, newSchool)

	# add a ZCatalog and a bunch of index info

	try:
		self.manage_addProduct['ZCatalog'].manage_addZCatalog('Catalog','ZCatalog for Tuttle')

	except:
		LOG("Tuttle", INFO, "There's already a ZCatalog called 'Catalog.'")

	else:
		# Student and teacher properties
		self.Catalog.manage_addIndex('FirstName', 'FieldIndex')
		self.Catalog.manage_addIndex('MiddleName', 'FieldIndex')
		self.Catalog.manage_addIndex('LastName', 'FieldIndex')
		self.Catalog.manage_addIndex('PreferredName', 'FieldIndex')
		self.Catalog.manage_addIndex('Team', 'FieldIndex')
		self.Catalog.manage_addIndex('Gender', 'FieldIndex')
		self.Catalog.manage_addIndex('Title', 'FieldIndex')
		# Course properties
		self.Catalog.manage_addIndex('CourseName', 'FieldIndex')
		self.Catalog.manage_addIndex('Subject', 'FieldIndex')
		self.Catalog.manage_addIndex('Level','FieldIndex')
		self.Catalog.manage_addIndex('Description', 'TextIndex')
		self.Catalog.manage_addIndex('Roster', 'TextIndex')


	return self.manage_main(self, REQUEST)

# web interface methods
manage_addSchoolInfoForm = DTMLFile("dtml/addSchoolInfoForm", globals())

class SchoolInfo(Folder):
	"""
	The SchoolInfo Class
	This class is the heart of a Tuttle Site.
	It contains the default site configuration options,
	as well as the basic "API" for returning student
	and teacher data.
	"""

	meta_type = "Tuttle School Site"



        ################################
	#                              #
	#  Site Configuration Options  #
	#                              #
	################################

        # These can also be changed from the SchoolInfo object's properties tab in the ZMI.

	# A list of the grades or levels in the school
	# e.g., in traditional (high) schools, GradeLevelList would be ['9','10','11','12']
	GradeLevelList=['9', '10','11','12']

	# If students and teachers are organized into teams, academies, etc.
	TeamList=['red','blue','yellow','black'] # these teams include teachers and students
	TeacherTeamList = TeamList + ['no team'] # additional "teams" for teachers only

	# These need to be updated in a few years...
	# Graduation Years
	GradYearList=["2003","2004","2005","2006","Unknown"]

	# These are all the subjects taught in the school.
	SubjectList=['English','Social Studies','Mathematics','Science','ESL',\
	'Health and Physical Education','Spanish','French','Art','Music']

	# Periods
	# Each period with a unique combination of classes across the school
	# needs a unique identifier.
	PeriodList=['A','B','C','D','E','F']


	# These aren't necessary, strictly speaking.  They just prevent some
	# errors from cropping up, since I've changed the TeacherClass properties along the way.
	Subject = ""
	Room = ""
	classNumber = 0
	Level = "Unknown"
	Period = ""
	Subject = ""
	Coursename = ""
	Description = ""

	Genders=('Male','Female')

	def __init__ (self, id, title,\
	district="Unknown School District", \
	superintendent="Unknown"):
		"Initialize the SchoolInfo object."
		self.id=id
		self.title=title # maps to SIF.SchoolInfo.SchoolName
		self.district=district
		self.superintendent=superintendent
		self.Days={}
		self.Modules=[]

	# Security boilerplate
        security = ClassSecurityInfo()

	# set up management tabs
	manage_options=Folder.manage_options[:3] + (
        {'label':'Configure Schedule','action':'manage_Schedule'},
	{'label':'Add Team','action':'manage_addTeam'},
	{'label':'Rename Teams','action':'manage_renameTeam'},
        ) + Folder.manage_options[3:]

	# declare properties
	_properties=(
	{'id':'title', 'type':'string'},
	{'id':'district', 'type':'string'},
	{'id':'superintendent','type':'string'},
	{'id':'GradeLevelList','type':'lines'},
	{'id':'TeacherTeamList','type':'lines'},
	{'id':'SubjectList','type':'lines'},
	{'id':'PeriodList','type':'lines'},
	{'id':'GradYearList','type':'lines'},
	{'id':'Genders','type':'lines'},
	)

	# Class level functions and (web) methods
	security.declareProtected('View', 'index_html')
	index_html=DTMLFile("dtml/indexSchoolInfo", globals())

	security.declareProtected('View management screens', 'manage_Schedule')
	manage_Schedule=DTMLFile("dtml/manageSchedule", globals())

	security.declareProtected('View management screens', 'manage_renameTeam')
	manage_renameTeam=DTMLFile("dtml/renameTeam", globals())

	security.declareProtected('Manage School', 'editSchoolInfo')
	def editSchoolInfo(self, title, district, superintendent,REQUEST):
		"Edit School Properties"
		self.title=title
		self.district=district
		self.superintendent=superintendent
		return self.index_html(self, REQUEST)

	####################
	#                  #
	#  Managing Teams  #
	#                  #
	####################

        # Making the teams folderish objects in the Tuttle rootleaves room for
        # additions in the future, but isn't very clean.  Teams aren't tightly
        # integrated into the data structure so that moving students around
        # and reorganizing teams does not cause major problems.

	security.declareProtected('Manage School', 'addTeam')
        def addTeam(self, id, title, REQUEST=None):
		"Adds a new team folder"
         	newTeam=Team(id, title)
       	  	self._setObject(id, newTeam)
         	if REQUEST is not None:
	        	return self.manage_main(self, REQUEST)

	security.declareProtected('Manage School', 'renameTeam')
	def renameTeam(self, oldname, newname, REQUEST=None, RESPONSE=None):
		"""Renames the team instance and changes teachers and students.
                Our administrators caused me a huge headache when they changed
                the team names just as I was trying to deploy this."""
		for tm in self.objectValues('Team Folder'):
			if tm.title == oldname:
				tm.title = newname
				tm.reindex_object()
		for st in self.student.objectValues('student'):
			if st.Team == oldname:
				st.Team = newname
				st.reindex_object()
		for tch in self.teacher.objectValues('teacher'):
			if tch.Team == oldname:
				tch.Team = newname
				tch.reindex_object()
		if REQUEST is not None:
			return self.manage_main(self, REQUEST)

	security.declareProtected('View management screens', 'manage_addTeam')
	manage_addTeam=DTMLFile("dtml/manageAddTeam", globals())


	##########################
	#                        #
	#  Setting the schedule  #
	#                        #
	##########################

	# Creating a data structure flexible enough to handle most
	# schedules a school is likely to create is a small nightmare.
	# Here's what we've got:
	#
	# A period is a unique team or schoolwide configuration of classes.
	# PeriodList is a list of the arbitrary names of all periods.
	#
	# A module is a defined block of time.
	# Modules is a list of dictionaries.
	# The keys are 'id', 'start' and 'end'.
	# The values for 'id' should be integers, starting
	# at 0 and going chronologically through the day.
	#
	# A day is a unique sequence of periods,
	# assumed to be within a calendar day.
	#
	# Days is a dictionary.
	# The keys are the arbitrary names of each day;
	# the values are tuples of the sequence of periods in that day.
	#
	# Whew!

	security.declareProtected('Manage School', 'addDay')
	def addDay(self, day, REQUEST=None):
		"Adds an entry for a day to the Day dictionary"
		thisDay = Day(day)
		self.Days[day] = thisDay
		for teamsched in self.objectValues('Team Folder'):
			teamsched.Days[day] = thisDay
			teamsched.__changed__(1)
		self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self, REQUEST)

	security.declareProtected('Manage School', 'delDay')
	def delDay(self, delID, REQUEST=None):
		"Delete some days"
		del self.Days[delID]
		for teamsched in self.objectValues('Team Folder'):
                        teamsched.Days[delID]
                        teamsched.__changed__(1)
		self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self.REQUEST)

	security.declareProtected('Manage School', 'addModules')
	def addModules(self, count, REQUEST=None):
		"Create a set of modules"
		if self.Modules == []:
			for x in range(count):
				thisModule={'id':x}
				self.Modules.append(thisModule)
			self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self.REQUEST)

	security.declareProtected('Manage School', 'editModule')
	def editModule(self, mod, start, end, REQUEST=None):
		"Edits  module start and end times"
		for x in self.Modules:
			if x['id']==mod:
				x['start']=start
				x['end']=end
		self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self.REQUEST)

	security.declareProtected('Manage School', 'delMods')
	def delMods(self, REQUEST=None):
		"Just clears all the modules"
		self.Modules=[]
		self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self.REQUEST)

	security.declareProtected('Manage School', 'setDayPeriods')
	def setDayPeriods(self, day, sched, REQUEST=None):
		"Sets a tuple of periods as a value in the Days dictionary"
		self.Days[day]=sched
		self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self.REQUEST)

	security.declareProtected('Manage School', 'delDayPeriods')
	def delDayPeriods(self, day, REQUEST=None):
		"Clears a day's periods"
		self.Days[day]=()
		self.__changed__(1)
		if REQUEST is not None:
			return self.manage_Schedule(self.REQUEST)

	#############################
	#                           #
	#  User Management Methods  #
	#                           #
	#############################


	security.declareProtected('Access Tuttle Methods', 'GenPasswd')
	def GenPasswd(self):
		"Generates a random 8 character password"
		# from ASPN's Python Cookbook (online)
		# submitted by Devin Leung
		chars = string.letters + string.digits
		newpasswd=""
		for i in range(8):
			newpasswd = newpasswd + choice(chars)
		return newpasswd

	security.declareProtected('Manage School', 'addNewTuttleUser')
	def addNewTuttleUser(self, UserID, roles, domains, REQUEST=None):
		"Creates a new Tuttle User if the user name does not exist already"
		if self.acl_users.getUser(UserID):
			return MessageDialog(
			title='User Already Exists',
			message='There is already a user with this name.',
			action='manage_main'
			)
		else:
			pwd = self.GenPasswd()
			confirm = pwd
			self.acl_users._addUser(UserID, pwd, confirm, roles, domains, REQUEST)
			if REQUEST is not None:
				return self.manage_main(self.REQUEST)



	######################
	#                    #
	# Tuttle API methods #
	#                    #
	######################
	security.declareProtected('View','quote')
	def quote (self):
		"Returns a quote from Harry Tuttle, from the movie Brazil, by Terry Gilliam"
		return non_sequitur.TuttleQuote()

	# this method is handy if you don't understand object-orientation :-0
	security.declareProtected('View', 'getInfo')
	def getInfo (self, role, userid, Method):
		"""This method allows you to make an arbitrary call on a
		student, teacher or administrator object.
		Role is 'student' or 'teacher' or 'administrator'
		It is bad object oriented form, but will
		save you time in a pinch"""
		call=""
		CodeString = "call = self." + role +  "." + userid + "." + Method
		exec CodeString
		return call
	
	security.declareProtected('View', 'getInfo')
	def getStudentObjectFromID (self, studid):
		"This returns a student object when passed its id"
		x = self.Catalog({'meta_type':'student', 'id':studid})
		z = x[0].getObject() # all these should have some error handling if more than one is found
		return z


	# these methods cover various formats for people's names

	security.declareProtected('View', 'getFullName')
	def getFullName (self, role, userid):
		"Returns 'First Middle Last'"
                x = self.Catalog({'meta_type':role, 'id':userid})
                z = x[0].getObject()
		first = z.FirstName
		middle = z.MiddleName
		last = z.LastName
		if middle:
			fullname=string.join([first,middle], " ")
		else:
			fullname=first
		fullname=string.join([fullname,last], " ")
		return fullname

	security.declareProtected('View', 'getNameLastFirst')
	def getNameLastFirst (self, role, userid):
		"Returns 'Last, Middle First'"
		lastfirst = ""
                x = self.Catalog({'meta_type':role, 'id':userid})
                z = x[0].getObject()
		first = z.FirstName
		middle = z.MiddleName
		last = z.LastName
		lastfirst = last + ", " + first
		if middle:
			lastfirst = lastfirst + " " + middle
		return lastfirst

	security.declareProtected('View', 'getFullerName')
	def getFullerName (self, role, userid):
		"Returns 'Prefix First Middle Last Suffix'"
		fullername = self.getFullName (role, userid)
		fullername = self.addPrefixSuffix (role, userid, fullername)
		return fullername

	# rewrite with ZCatalog queries
	security.declareProtected('View', 'addPrefixSuffix')
	def addPrefixSuffix (self, role, userid, name):
		"Adds prefix and suffix"
		prefix = ""
                suffix = ""
                x = self.Catalog({'meta_type':role, 'id':userid})
                z = x[0].getObject()
		if role != 'student':
			prefix = z.prefix
		if prefix :
			name=string.join([prefix, name], " ")
		try:
                        suffix = z.suffix
			name=string.join([name, suffix], " ")
		except:
                        pass
                return name

	security.declareProtected('View', 'getFullestName')
	def getFullestName (self, role, userid):
		"""Returns name in form
		'Prefix FirstName MiddleName (PreferredName) LastName Suffix'"""
                x = self.Catalog({'meta_type':role, 'id':userid})
                z = x[0].getObject()
                fullestname = self.getFullName(role, userid)
		pref = z.PreferredName
		if pref:
			pref = string.join(['(', pref, ')'],"")
			pref = [pref]
			fullestname=string.split(fullestname, ' ',)
			fullestname=fullestname[:-1] + pref + fullestname[-1:]
			fullestname=string.join(fullestname, " ")
		fullestname = self.addPrefixSuffix(role, userid, fullestname)
		return fullestname

	security.declareProtected('View', 'getTeamMembers')
	def getTeamMembers (self, role, team="no team"):
		'Returns a list of the members of a team, or all members'
		if team == "no team" or team == "No Team" or team == "No_Team" or team == "No":
			x = self.Catalog({'meta_type':'student'})
		else:
			x = self.Catalog({'meta_type':'student','Team':team})
		z = []
		for y in x:
			z.append(y.getObject())
		return z

	security.declareProtected('View', 'getTeams')
	def getTeams(self):
		"Returns a list of teams"
		x = self.Catalog({'meta_type':'Team Folder'})
		z = []
		for y in x:
			z.append(y.getObject())
		return z

	# Cookie management
	security.declareProtected('Access Tuttle Methods', 'setTM')
	def setTM(self,REQUEST,RESPONSE):
		'Sets a team cookie (for class roster form)'
		RESPONSE.setCookie('TM',REQUEST.TM)
		# I really have to learn to do this right to make it more generalized.
		return RESPONSE.redirect(REQUEST.URL1 + '/manage_main')



	# these don't seem to be working, by the way
	security.setPermissionDefault('View', ['Anonymous', 'Authenticated'])
	security.setPermissionDefault('Access Contents Information',['Manager','administrator'])
	security.setPermissionDefault('View management screens', ['Manager','administrator'])
	security.setPermissionDefault('Access Tuttle Methods', ['Manager', 'administrator', 'teacher'])
	security.setPermissionDefault('Manage School', ['Manager','Owner','administrator'])

InitializeClass(SchoolInfo)

