#####################
#  StudentPersonal  #
#####################

##########
#
#  Copyright 2002, Tom Hoffman.
# 
#  This file is part of Tuttle.
#
#  Tuttle is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  Tuttle is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Tuttle (see COPYING.txt); if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##########

from OFS.Folder import Folder
from Globals import DTMLFile, InitializeClass
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from AccessControl import ClassSecurityInfo
from pdfReportLab import pdfStudSchedule

class StudentPersonal(CatalogAware, Folder):
	"The basic container for a student's objects, with some basic attributes"
	meta_type = "student"
	
	_properties=(
	{'id':'title','type':'string'},
	{'id':'FirstName','type':'string'},
	{'id':'LastName','type':'string'},
	{'id':'MiddleName','type':'string'},
	{'id':'PreferredName','type':'string'},
	{'id':'Suffix','type':'string'},
	{'id':'Gender','type':'selection','select_variable':'Genders'},
	{'id':'GradeLevel','type':'selection','select_variable':'GradeLevelList'},
	{'id':'GradYear','type':'selection','select_variable':'GradYearList'},
	{'id':'Team','type':'string'},
	)
	
	manage_options=Folder.manage_options[:3] + (
        {'label':'Class Schedule','action':'manage_StudentClassSchedule'},
        {'label':'.pdf Schedule','action':'manage_pdfSchedule'}
        ) + Folder.manage_options[3:]

	def __init__ (self, id, FirstName, LastName, MiddleName="", PreferredName="", \
	Suffix="", Gender="?", GradeLevel="?", GradYear="?", Team="?", REQUEST=None):
		self.id=id
		self.editStudentPersonal(FirstName, LastName, MiddleName, PreferredName, \
		Suffix, Gender, GradeLevel, GradYear,Team)
	
        # set up security boilerplate
        security = ClassSecurityInfo()
#       security.setDefaultAccess("deny")
#       __roles__ = ()
	
	security.declareProtected('Manage School', 'editStudentPersonal')
	def editStudentPersonal (self, FirstName, LastName, MiddleName, PreferredName, \
	Suffix, Gender, GradeLevel, GradYear, Team, REQUEST=None):
		"Method for updating student personal data"
		self.FirstName=FirstName
		self.MiddleName=MiddleName
		self.LastName=LastName
		self.PreferredName=PreferredName
		if self.title[-7:] != 'Records':
			if self.PreferredName != "":
				self.title=PreferredName+"\'s Records" 
			else:
				self.title=FirstName+"\'s Records"             
		self.Suffix=Suffix
		self.Gender=Gender
		self.GradeLevel=GradeLevel
		self.GradYear=GradYear
		self.Team=Team
		self.reindex_object()
		if REQUEST is not None:
			return self.index_html(self,REQUEST) 
	

	security.declareProtected('Access Student Schedule', 'getSchedule')	
	def getSchedule(self):
		"Returns a list of classes with this student on the roster"
		x = self.Catalog({'Roster':self.id, 'meta_type':'Teacher Class'})
		z=[]
		for y in x:
			z.append(y.getObject())
		return z
	
	security.declareProtected('Access Student Schedule', 'ScheduleGrid')
	def ScheduleGrid(self):
		"""gets the sequence of periods, puts in the right classes,
		and returns a handy list for formatting horizontally in a table"""
		x = self.getScheduleGrid()
		y = self.fillScheduleGrid(x)
		return y




	security.declareProtected('Access Student Schedule', 'getScheduleGrid')
	def getScheduleGrid(self):
		"""This is the surprisingly complex method necessary to lay
		out the sequence of periods in a schedule grid."""
		# look for the student's team
		teamQuery = self.Catalog.searchResults({'meta_type':'Team Folder','title':self.Team})
		# turn the resulting map into a list
		# if there is a way around this I'd like to know
		z = []
		sched = {}
		for y in teamQuery:
			z.append(y.getObject()) # to do: error message if len(z) > 1
		if z == []:                     # i.e., no match
			for y in self.Days:     # use the school schedule
				z.append(y.getobject())  # this could be cleaned up a bit
		sched = z[0].Days 		# use the team (the first one you found) schedule
		allDays = sched.keys()	        # strip out the days so we can
		allDays.sort() 			# sort them
		FullMonty = []
		OneRow = []
		mods = len(self.Modules)
		for x in range(mods):              # run this loop once for each mod
			for y in allDays:          # loop through each day 
				z = sched[y][x]    # get the period in the right module & day
				OneRow.append(z)
			FullMonty.append(OneRow)
			OneRow=[]
		return FullMonty
		
	security.declareProtected('Access Student Schedule', 'fillScheduleGrid')
	def fillScheduleGrid(self,grid):
		"Replaces period designations with class info for a student"
		result = []
		row = []
		entry = []
		classes = self.getSchedule()
		for x in grid:					# gets a row (x) from the grid
			for y in x:				# gets a period (y) from the row
				entry = [y, 'Unknown Course', 'Unknown Teacher', 'Unknown Room']
				found = 0
				for z in classes:		# iterates (z) through the classes
					for a in z.Period:	# each period (a) a class meets
						if a == y:  
							entry = [y, z.CourseName, z.LastName, z.Room]
							found = found + 1
				if found > 1:
					entry = ["Error:", "You are", "double", "booked"]
				row.append(entry)
			result.append(row)
			row = []
		return result




	security.declareProtected('Access Student Schedule', 'printSchedule')
	def printSchedule(self):
		"Gets a pdf version of the schedule"
		x = pdfStudSchedule(self)
		return x
	
	# Web Presentation Methods
	security.declareProtected('View', 'index_html')
	index_html=DTMLFile("dtml/indexStudentPersonal",globals())
	security.declareProtected('Access Student Schedule', 'manage_StudentClassSchedule')
	manage_StudentClassSchedule=DTMLFile("dtml/StudentClassSchedule", globals())
        security.declareProtected('Access Student Schedule', 'manage_pdfSchedule')
        manage_pdfSchedule=DTMLFile("dtml/pdfSchedule", globals())

	security.setPermissionDefault('Access Student Schedule',\
	['Manager', 'teacher', 'administrator', 'guardian', 'Owner'])

InitializeClass(StudentPersonal)
