############
#  TeacherClass  #
############

##########
#
#  Copyright 2002, Tom Hoffman.
#
#  This file is part of Tuttle.
#
#  Tuttle is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  Tuttle is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Tuttle (see COPYING.txt); if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##########

from OFS.Folder import Folder
from Globals import DTMLFile, InitializeClass
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from AccessControl import ClassSecurityInfo

class TeacherClass(CatalogAware, Folder):
	"Make one of these folderish objects for each class a teacher has"

	meta_type = "Teacher Class"

	Roster = []

	_properties=(
	{'id':'title', 'type':'string'},
	{'id':'CourseName', 'type':'string'},
	{'id':'Subject', 'type':'selection','select_variable':'SubjectList'},
	{'id':'Description','type':'text'},
	{'id':'Level','type':'multiple selection','select_variable':'GradeLevelList'},
	{'id':'Period','type':'multiple selection','select_variable':'PeriodList'},
	{'id':'Room','type':'string'},
	{'id':'Roster','type':'lines'},
        {'id':'classNumber','type':'int'}
	)

	# set up our management tabs
	# KNOWN BUG: 'View' isn't displayed for no apparent reason.  index_html is viewable.
	manage_options=Folder.manage_options[:3] + (
	{'label':'Create Roster','action':'manage_ClassRosterForm'},
	{'label':'Edit Roster','action':'manage_editRoster'},
	) + Folder.manage_options[3:]

	def __init__ (self, id, CourseName="" , Subject="", Level=[], Period=[], Description="", REQUEST=None):
		self.id=id
		self.editTeacherClass(CourseName, Subject, Level, Period, Description)
	
	# set up security boilerplate
        security = ClassSecurityInfo()
        security.setDefaultAccess("deny")
        __roles__ = ()

	security.declareProtected('Manage School', 'editTeacherClass')
	def editTeacherClass(self, CourseName, Subject, Level, Period, Description, REQUEST=None):
		"Method for updating class information "
		self.title=CourseName + " Class Folder"
		self.CourseName=CourseName
		self.Subject=Subject
		self.Level=Level
		self.Period=Period
		self.Description=Description
		self.reindex_object()
		if REQUEST is not None:
			return self.index_html(self,REQUEST)

	security.declareProtected('Manage Roster', 'manageClassRoster')
	def manageClassRoster(self, rosterAdds=[], REQUEST=None):
		"Sets a list of names as a roster"
		self.Roster = rosterAdds
		self.reindex_object()
		if REQUEST is not None:
			return self.manage_ClassRosterForm(self, REQUEST)
			
	security.declareProtected('Manage Roster', 'rosterAdd')
	def rosterAdd(self, rosterAdds=[], REQUEST=None, RESPONSE=None):
		"Adds a list of names to a roster"
		for name in rosterAdds:
			self.Roster.append(name)
		self.__changed__(1)
		self.reindex_object()
		if REQUEST is not None:
			return RESPONSE.redirect(REQUEST.URL1 + '/manage_editRoster')
			
	security.declareProtected('Manage Roster', 'rosterDel')
	def rosterDel(self, rosterDels=[], REQUEST=None, RESPONSE=None):
		"Deletes a list of names from a roster"
		for name in rosterDels:
			self.Roster.remove(name)
		self.__changed__(1)
		self.reindex_object()
		if REQUEST is not None:
			return RESPONSE.redirect(REQUEST.URL1 + '/manage_editRoster')

	# Web Presentation Methods
	security.declareProtected('View', 'index_html')
	index_html=DTMLFile("dtml/indexTeacherClass",globals())
	
	security.declareProtected('Manage School', 'addTeacherClass')
	addTeacherClass=DTMLFile("dtml/addTeacherClassForm",globals())
	
	security.declareProtected('Manage Roster', 'manage_ClassRosterForm')
	manage_ClassRosterForm=DTMLFile("dtml/manageClassRoster",globals()) 
	
	security.declareProtected('Manage Roster', 'manage_editRoster')
	manage_editRoster = DTMLFile("dtml/editRoster", globals())
	
	security.setPermissionDefault('Manage Roster', ['Manager','administrator'])

InitializeClass(TeacherClass)
