####################
#  TeachersFolder  #
####################

##########
#
#  Copyright 2002, Tom Hoffman.
# 
#  This file is part of Tuttle.
#
#  Tuttle is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  Tuttle is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Tuttle (see COPYING.txt); if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##########

from OFS.Folder import Folder
from TeacherPersonal import TeacherPersonal
from Globals import DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo
import string



class TeachersFolder(Folder):
	"These objects contain individual teachers folders."
	
	meta_type="Teachers Folder"

	_properties=(
	{'id':'title', 'type':'string'},
        {'id':'LastClassNumber', 'type':'int'},
	)

	manage_options=Folder.manage_options[:3] + (
	{'label':'Add Teacher','action':'manage_addTeacherPersonalForm'},
	) + Folder.manage_options[3:]


	def __init__ (self,id,title):
		self.id=id
		self.title=title
                self.LastClassNumber = 0

        # set up security boilerplate
        security = ClassSecurityInfo()
#        security.setDefaultAccess("deny")
#       __roles__ = ()


	# methods to add, edit and retrieve TeacherPersonal instances

	security.declareProtected('Manage School', 'addTeacherPersonal')
	def addTeacherPersonal(self, id, FirstName, LastName, MiddleName, PreferredName, \
	Prefix, Suffix, Gender, Team, REQUEST=None):
		"Method to add a TeacherPersonal object to a TeacherFolder"
		newTeacherPersonal=TeacherPersonal(id, FirstName, LastName, MiddleName, \
		PreferredName, Prefix, Suffix, Gender, Team,)
		newTeacherPersonal.manage_setLocalRoles(id,['Owner'])
		self._setObject(id, newTeacherPersonal)
		self.addNewTuttleUser(id, ['teacher'], [])
		if REQUEST is not None:
			return self.manage_main(self, REQUEST)

        security.declareProtected('Access Tuttle Methods', 'takeNumber')
        def takeNumber(self):
                "Pass an id number to a class and increment it one step"
                self.LastClassNumber = self.LastClassNumber + 1
                x = self.LastClassNumber
                return x

        def addClassIDs(self):
                """Run this on a TeachersFolder to give ID numbers to classes.
                In this case, each class gets a new id at the end of the year
                or semester, so don't think of it as the equivalent of 'Physics 101,'
                for example.  This generally shouldn't be needed or used."""
                result = "Answer: "
                for T in self.objectValues('teacher'):
                        for C in T.objectValues('Teacher Class'):
                                try:
                                        result = result + str(C.classNumber) + " is OK, "
                                except:
                                        C.classNumber = self.takeNumber()
                                        C.reindex_object
                                        result = result + C.id + " -- " + str(C.classNumber) + "\n"
                return result



	# web presentation methods
	security.declareProtected('View management screens', 'manage_addTeacherPersonalForm')
	manage_addTeacherPersonalForm=DTMLFile("dtml/addTeacherPersonalForm",globals())

InitializeClass(TeachersFolder)
