##########
#  Team  #
##########

##########
#
# Copyright 2002, Tom Hoffman.
#
# This file is part of Tuttle.
#
# Tuttle is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Tuttle is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Tuttle; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##########

from OFS.Folder import Folder
from Globals import DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo
from Products.ZCatalog.CatalogPathAwareness import CatalogAware
from Acquisition import Implicit


class Team(CatalogAware,Folder):
	"""This object contains some properties for individual teams.
	Currently, it doesn't do much-- doesn't contain a list of team
	members, doesn't contain member objects.  It is a convenient
	place to put individual team Days dictionary, so each team
	can have the same schedule.  It seems likely there will be similar
	uses in the future.  In general, we don't want to load functionality
	into this, because we want to allow easy disolution and reorganization 
	of teams without breaking related data."""

	meta_type="Team Folder"

	manage_options=Folder.manage_options[:3] + (
	{'label':'Edit Team Schedule','action':'manage_TeamSchedule'},
	) + Folder.manage_options[3:]


	def __init__(self,id,title):
		self.id=id
		self.title=title
		self.Days={}

        # set up security boilerplate
	security = ClassSecurityInfo()
#      security.setDefaultAccess("deny")
#	__roles__ = ()
			
        security.declareProtected('Manage School', 'setDayPeriods')
        def setDayPeriods(self, day, sched, REQUEST=None):
                "Sets a tuple of periods as a value in the Days dictionary"
                self.Days[day]=sched
                self.__changed__(1)
                if REQUEST is not None:
                        return self.manage_TeamSchedule(self.REQUEST)

        security.declareProtected('Manage School', 'delDayPeriods')
        def delDayPeriods(self, day, REQUEST=None):
                "Clears a day's periods"
                self.Days[day]=()
                self.__changed__(1)
                if REQUEST is not None:
                        return self.manage_TeamSchedule(self.REQUEST)

	security.declareProtected('Manage School', 'resetSchedule')
	def resetSchedule(self, RESPONSE, REQUEST):
		"Sets the team's Day to the school Day"
		self.Days=REQUEST.PARENTS[1].Days
		return self.manage_TeamSchedule(self,REQUEST)

	# web methods
	manage_TeamSchedule=DTMLFile("dtml/manageTeamSchedule",globals())
