##################
#  pdfReportLab  #
##################

##########
#
#  Copyright 2002, Tom Hoffman.
#
#  This file is part of Tuttle.
#
#  Tuttle is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  Tuttle is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with Tuttle (see COPYING.txt); if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##########

import os
from reportlab.lib.pagesizes import letter
from reportlab.platypus import SimpleDocTemplate, Paragraph, Spacer, Table, PageBreak
from reportlab.rl_config import defaultPageSize
from reportlab.lib.units import inch
from reportlab.lib import styles
from reportlab.lib.colors import black

# This isn't very well integrated yet.  It dumps the pdf's into $INSTANCE_HOME/pdf.
# I use a Local File System object to access them at the moment.  They should just
# go into the ZODB, but I haven't set that up yet.

def pdfStudSchedule(self):
	"Generates a pdf version of a student's schedule"

	os.chdir(INSTANCE_HOME + '/pdf')

	PDFname = self.id+'.pdf'

	report = SimpleDocTemplate(PDFname, pagesize=letter)

	styles=getSchedStyleSheet()
	styleNormal=styles['Normal']
	styleTitle=styles['Title']
	styleH2=styles['Heading2']
	styleH3=styles['Heading3']

        feedback = "Saved .pdf schedule for " + self.id + " to pdf folder on the filesystem."

	# The content of the schedule goes into this list--schedule

	schedule = []
	# add student name
	studentName = self.getFullestName('student',self.id)
	schedule.append(Paragraph(studentName, styleTitle))
	schedule.append(Paragraph('Team: ' + self.Team, styleH2))
	schedule.append(Paragraph('Level: ' + self.GradeLevel, styleH2))

	instructions = """Please check the list of classes and schedule grid for mistakes.  If there is a missing class, enter the class information in the correct space.  The teacher's name is most important.  Cross out classes you don't have."""
	schedule.append(Paragraph(instructions, styleNormal))

	# print list of classes
	schedule.append(Paragraph('List of Classes:', styleH3))
	x = self.getSchedule()
	x.sort()
	for classes in x:
		try:
			for cl in classes:
				html2platypus(cl.CourseName)
				schedule.append(Paragraph(cl.Period + "(DUPLICATE FOR PERIOD): " + cl.Subject + "--" + cl.CourseName + ", " + cl.LastName, styleNormal))
		except:
			html2platypus(classes.CourseName)
			schedule.append(Paragraph(repr(classes.Period) + ": " + classes.Subject + "--" + classes.CourseName + ", " + classes.LastName, styleNormal))
	schedule.append(Spacer(0,.5*inch))
	# make the table
	data = self.ScheduleGrid()
	for row in data:
		count = 0
		newPeriod=[]
		for period in row:
			newPeriod =  period[0] + '\n' + period[1] + '\n' + period[2] + '\n' + period[3] + '\n \n'
			row[count] = newPeriod
			count = count + 1

	top = ['1A','2E','3A','4E']
	data.insert(0, top)
	t = Table(data, style=[('GRID',(0,0),(-1,-1),1,black)], colWidths=[1.5*inch, 1.5*inch, 1.5*inch, 1.5*inch])
	schedule.append(t)

	# OK, export the damn thing...
	report.build(schedule, onFirstPage=myFirstPage, onLaterPages=myLaterPages)
	return feedback

def pdfAllSchedules(self):
	"Generates a pdf version of all students' schedules"
	# forgive the repetition of code, I'll split the pieces apart for
	# greater effiency and elegance later
	os.chdir(INSTANCE_HOME + '/pdf')

	PDFname = 'all_schedules.pdf'

	report = SimpleDocTemplate(PDFname, pagesize=letter)

	styles=getSchedStyleSheet()
	styleNormal=styles['Normal']
	styleTitle=styles['Title']
	styleH2=styles['Heading2']
	styleH3=styles['Heading3']

	# The content of the schedule goes into this list--schedule
        feedback = ""
	schedule = []

	for ST in self.objectValues('student'):

		feedback = feedback + "Saved .pdf schedule for " + self.id + " to pdf folder on the filesystem.<br>"

                # add student name
		studentName = ST.getFullestName('student',ST.id)
		schedule.append(Paragraph(studentName, styleTitle))
		schedule.append(Paragraph('Team: ' + ST.Team, styleH2))
		schedule.append(Paragraph('Level: ' + ST.GradeLevel, styleH2))

		instructions = """Please check the list of classes and schedule grid for mistakes.  If there is a missing class, enter the class information in the correct space.  The teacher's name is most important.  Cross out classes you don't have."""
		schedule.append(Paragraph(instructions, styleNormal))

		# print list of classes
		schedule.append(Paragraph('List of Classes:', styleH3))
		x = ST.getSchedule()
		x.sort()
		for classes in x:
			try:
				for cl in classes:
					cl.CourseName = html2platypus(cl.CourseName)
					schedule.append(Paragraph(cl.Period + "(DUPLICATE FOR PERIOD): " + cl.Subject + "--" + cl.CourseName + ", " + cl.LastName, styleNormal))
			except:
				classes.CourseName = html2platypus(classes.CourseName)
				schedule.append(Paragraph(repr(classes.Period) + ": " + classes.Subject + "--" + classes.CourseName + ", " + classes.LastName, styleNormal))
		schedule.append(Spacer(0,.5*inch))
		# make the table
		data = ST.ScheduleGrid()
		for row in data:
			count = 0
			newPeriod=[]
			for period in row:
				newPeriod =  period[0] + '\n' + period[1] + '\n' + period[2] + '\n' + period[3] + '\n \n'
				row[count] = newPeriod
				count = count + 1

		top = ['1A','2E','3A','4E']
		data.insert(0, top)
		t = Table(data, style=[('GRID',(0,0),(-1,-1),1,black)], colWidths=[1.5*inch, 1.5*inch, 1.5*inch, 1.5*inch])
		schedule.append(t)
		schedule.append(PageBreak())

	# OK, export the damn thing...
	report.build(schedule, onFirstPage=myFirstPage, onLaterPages=myLaterPages)
	return feedback


def html2platypus(text):

	# this changes html tags to the xml markup that ReportLab's Platypus page layout engine
	# supported tags:  <p> to <para> (I think...)
	# other changes: '&' to 'and'

	i=0
	for c in text:
		if c == '&':
			text=text[:i]+'and'+text[i+1:]
			i=i+2
		if c == "<":
			if text[i+1]=='p' and text [i+2]==">":
				text=text[:i+1]+'para'+text[i+2:]
			if text[i+1]=="/":
				if text[i+2]=="p" and text[i+3]==">":
					text=text[:i+2]+'para'+text[i+3:]
		i=i+1
	return text


def myFirstPage(canvas,report):
	canvas.saveState()
	canvas.setFont('Times-Roman',9)

	canvas.restoreState()

def myLaterPages(canvas,report):
	canvas.saveState()
	canvas.setFont('Times-Roman',9)

	canvas.restoreState()


def getSchedStyleSheet():
    """Returns a stylesheet object"""
    stylesheet = styles.StyleSheet1()

    stylesheet.add(styles.ParagraphStyle(name='Normal',
                                  fontName='Times-Roman',
                                  fontSize=10,
                                  leading=12)
                   )

    stylesheet.add(styles.ParagraphStyle(name='BodyText',
                                  parent=stylesheet['Normal'],
                                  spaceBefore=6)
                   )
    stylesheet.add(styles.ParagraphStyle(name='Italic',
                                  parent=stylesheet['Normal'],
                                  fontName = 'Times-Italic')
                   )

    stylesheet.add(styles.ParagraphStyle(name='Bold',
                                  parent=stylesheet['BodyText'],
                                  fontName = 'Times-Bold')
                   )

    stylesheet.add(styles.ParagraphStyle(name='Heading1',
                                  parent=stylesheet['Normal'],
                                  fontName = 'Times-Bold',
                                  fontSize=18,
                                  leading=22,
                                  spaceAfter=6),
                   alias='h1')

    stylesheet.add(styles.ParagraphStyle(name='Title',
                                  parent=stylesheet['Normal'],
                                  fontName = 'Times-Bold',
                                  fontSize=24,
                                  leading=22,
                                  spaceBefore=24,
				  spaceAfter=6),
                   alias='title')

    stylesheet.add(styles.ParagraphStyle(name='Heading2',
                                  parent=stylesheet['Normal'],
                                  fontName = 'Times-Bold',
                                  fontSize=14,
                                  leading=18,
                                  spaceBefore=6,
                                  spaceAfter=3),
                   alias='h2')

    stylesheet.add(styles.ParagraphStyle(name='Heading3',
                                  parent=stylesheet['Normal'],
                                  fontName = 'Times-BoldItalic',
                                  fontSize=12,
                                  leading=14,
                                  spaceBefore=9,
                                  spaceAfter=3),
                   alias='h3')

    stylesheet.add(styles.ParagraphStyle(name='Code',
                                  parent=stylesheet['Normal'],
                                  fontName='Courier',
                                  fontSize=12,
                                  leading=8.8,
                                  firstLineIndent=0,
                                  leftIndent=36,
				  spaceBefore=3,
				  spaceAfter=6))


    return stylesheet
